@@ -1,13 +1,17 @@
 import itertools
 import logging
 
+from itertools import product
+
 import numpy as np
 from numpy.testing._private.utils import raises
 import pandas as pd
 import scipy as sp
 
+from scipy import linalg
+
 from .eval import EvalEnvironment
-from .terms import ModelTerms
+from .terms import ModelTerms, InterceptTerm
 from .model_description import model_description
 from .utils import flatten_list
 
@@ -140,6 +144,7 @@ class CommonEffectsMatrix:
             self.terms_info[key] = {k: v for k, v in d[key].items() if k != "value"}
             delta = d[key]["value"].shape[1]
             self.terms_info[key]["cols"] = slice(start, start + delta)
+            self.terms_info[key]["full_names"] = self.get_term_full_names(key)
             start += delta
 
     def as_dataframe(self):
@@ -154,7 +159,7 @@ class CommonEffectsMatrix:
         term = self.terms_info[name]
         _type = term["type"]
         if _type == "Intercept":
-            return "Intercept"
+            return ["Intercept"]
         elif _type in ["numeric", "call"]:
             return [name]
         elif _type == "interaction":
@@ -162,8 +167,11 @@ class CommonEffectsMatrix:
         elif _type == "categoric":
             # "levels" is present when we have dummy encoding (not just a vector of 0-1)
             if "levels" in term.keys():
-                # drops first level because of full-rank matrix
-                return [f"{name}[{level}]" for level in term["levels"][1:]]
+                # Ask if encoding is "full" or "reduced"
+                if term["encoding"] == "full":
+                    return [f"{name}[{level}]" for level in term["levels"]]
+                else:
+                    return [f"{name}[{level}]" for level in term["levels"][1:]]
             else:
                 return [f"{name}[{term['reference']}]"]
 
@@ -195,8 +203,8 @@ class GroupEffectsMatrix:
     Parameters
     ----------
 
-    terms : ModelTerms
-        An ModelTerms object containing terms for the group effects of the model.
+    terms : list
+        A list of GroupSpecTerm objects.
     data: pandas.DataFrame
         The data frame where variables are taken from
     eval_env: EvalEnvironment
@@ -220,44 +228,80 @@ class GroupEffectsMatrix:
         Z = []
         self.terms_info = {}
         for term in self.terms:
-            d = term.eval(self.data, self.eval_env)
-            Zi = d["Zi"]
-            delta_row = Zi.shape[0]
-            delta_col = Zi.shape[1]
-            Z.append(Zi)
-            self.terms_info[term.to_string()] = {
-                "type": d["type"],
-                "groups": d["groups"],
-                "idxs": (
+
+            encoding = True
+            if not isinstance(term.expr, InterceptTerm):
+                for term_ in self.terms:
+                    if term_.factor == term.factor and isinstance(term_.expr, InterceptTerm):
+                        encoding = False
+            d = term.eval(self.data, self.eval_env, encoding)
+
+            if d["type"] == "categoric":
+                levels = d["levels"] if d["encoding"] == "full" else d["levels"][1:]
+                for idx, level in enumerate(levels):
+                    Xi = np.atleast_2d(d["Xi"][:, idx]).T
+                    Ji = d["Ji"]
+                    Zi = linalg.khatri_rao(Ji.T, Xi.T).T
+                    delta_row = Zi.shape[0]
+                    delta_col = Zi.shape[1]
+                    Z.append(Zi)
+                    term_name = term.to_string(level)
+                    self.terms_info[term_name] = {
+                        "type": "categoric",
+                        "Xi": Xi,
+                        "Ji": Ji,
+                        "groups": d["groups"],
+                        "encoding": d["encoding"],
+                        "levels": d["levels"],
+                        "reference": d["reference"],
+                        "full_names": [f"{term_name}[{group}]" for group in d["groups"]],
+                    }
+                    self.terms_info[term_name]["idxs"] = (
+                        slice(start_row, start_row + delta_row),
+                        slice(start_col, start_col + delta_col),
+                    )
+                    start_row += delta_row
+                    start_col += delta_col
+            else:
+                Zi = d["Zi"]
+                delta_row = Zi.shape[0]
+                delta_col = Zi.shape[1]
+                Z.append(Zi)
+                term_name = term.to_string()
+                self.terms_info[term_name] = {k: v for k, v in d.items() if k != "Zi"}
+                self.terms_info[term_name]["idxs"] = (
                     slice(start_row, start_row + delta_row),
                     slice(start_col, start_col + delta_col),
-                ),
-            }
-            if d["type"] == "categoric":
-                self.terms_info[term.to_string()]["levels"] = d["levels"]
-                self.terms_info[term.to_string()]["reference"] = d["reference"]
-            start_row += delta_row
-            start_col += delta_col
+                )
+                self.terms_info[term_name]["full_names"] = self.get_term_full_names(term_name)
+                start_row += delta_row
+                start_col += delta_col
+
         # Stored in Compressed Sparse Column format
-        self.design_matrix = sp.sparse.block_diag(Z).tocsc()
+        if Z:
+            self.design_matrix = sp.sparse.block_diag(Z).tocsc()
+        else:
+            self.design_matrix = np.zeros((0, 0))
 
     def get_term_full_names(self, name):
         # Always returns a list
         term = self.terms_info[name]
         _type = term["type"]
-        if _type == "Intercept":
-            return "Intercept"
-        elif _type in ["numeric", "call"]:
-            return [name]
+        if _type in ["Intercept", "numeric", "call"]:
+            return [f"{name}[{group}]" for group in term["groups"]]
         elif _type == "interaction":
             return interaction_label(term)
         elif _type == "categoric":
-            # "levels" is present when we have dummy encoding (not just a vector of 0-1)
+            # not used anymore?
             if "levels" in term.keys():
-                # drops first level because of full-rank matrix
-                return [f"{name}[{level}]" for level in term["levels"][1:]]
+                # Ask if encoding is "full" or "reduced"
+                if term["encoding"] == "full":
+                    products = product(term["levels"], term["groups"])
+                else:
+                    products = product(term["levels"][1:], term["groups"])
             else:
-                return [f"{name}[{term['reference']}]"]
+                products = product([term["reference"]], term["groups"])
+            return [f"{name}[{p[0]}|{p[1]}]" for p in products]
 
     def __getitem__(self, term):
         if term not in self.terms_info.keys():
@@ -358,7 +402,7 @@ def term_str(term):
                 vars.append(f"    {k}: {{" + ", ".join(str_l) + "}")
         x = "type=interaction, vars={\n" + ",\n".join(vars) + "\n  }"
     else:
-        x = ", ".join([k + "=" + str(v) for k, v in term.items()])
+        x = ", ".join([k + "=" + str(v) for k, v in term.items() if k not in ["Xi", "Ji"]])
     return x
 
 
@@ -371,7 +415,11 @@ def interaction_label(x):
             colnames.append([k])
         if v["type"] == "categoric":
             if "levels" in v.keys():
-                colnames.append([f"{k}[{level}]" for level in v["levels"][1:]])
+                # ask whether encoding is full or reduced
+                if v["encoding"] == "full":
+                    colnames.append([f"{k}[{level}]" for level in v["levels"]])
+                else:
+                    colnames.append([f"{k}[{level}]" for level in v["levels"][1:]])
             else:
                 colnames.append([f"{k}[{v['reference']}]"])
 
